#ifndef _POLICY_H
#define _POLICY_H

template <typename T>
class UnsafeRefCounter
{
public:
	UnsafeRefCounter() : d_refcount(T(0)){}
	
	void AddRef()
	{
		++d_refcount;
	}
	void Release()
	{
		--d_refcount;
	}
	bool active() const
	{
		return d_refcount != T(0);
	}
private:
	T d_refcount;
};

template <unsigned int BlockSize> 
class PoolProxy
{
public:
	template <class T>
	class PoolAllocation
	{
		struct Block
		{
			Block(Block* next) : d_next(next)
			{
				for (unsigned int i = 0; i < BlockSize - 1; ++i)
					d_slots[i].d_object = &d_slots[i + 1];
				d_slots[BlockSize - 1].d_object = 0;
			}
			~Block()
			{
				if (d_next) delete d_next;
			}
			Block* d_next;
			T d_slots[BlockSize];
		};

		class Pool
		{
			friend class SPIterator;
		public:
			Pool() : d_block_list(new Block(0)), d_free_list(&d_block_list->d_slots[0]){}
			~Pool()
			{
				delete d_block_list;
			}
			T* Allocate()
			{
				if (!d_free_list)
				{
					d_block_list = new Block(d_block_list);
					d_free_list = &d_block_list->d_slots[0];
				}
				T* p = d_free_list;
				d_free_list = static_cast<T*>(p->d_object);
				return p;
			}

			void Deallocate(T* p)
			{
				p->d_object = d_free_list;
				d_free_list = p;
				p->d_size = 0;
			}
		private:
			Block* d_block_list;
			T* d_free_list;
		};

		static Pool& pool()
		{
			static Pool d_pool = Pool();
			return d_pool;
		}
		
	public:
		class SPIterator
		{
		public:
			SPIterator() : d_slot(0), d_block(pool().d_block_list){}
			void operator++()
			{
				while (d_block)
				{
					if (d_slot >= BlockSize)
					{
						if (!(d_block = d_block->d_next))
							break;
						d_slot = 0;
					}
					if (d_block->d_slots[d_slot].d_size)
						break;
					d_slot++;
				}
			}
			T* operator()()
			{
				T* p = &d_block->d_slots[d_slot++];
				this->operator++();
				return p;
			}
			operator const void*() const
			{
				return d_block;
			}
		private:
			unsigned int d_slot;
			Block* d_block;
		};

		static void* operator new(size_t)
		{
			return pool().Allocate();
		}
		static void operator delete(void* p)
		{
			pool().Deallocate(static_cast<T*>(p));
		}
		static SPIterator iterator()
		{
			return SPIterator();
		}
	};
};


template <unsigned long SpaceSize>
class MarkCompactObjectAllocator
{
public:
	template <class T, template <class> class GC>
	class ObjectAllocator
	{
	public:
		static void* Allocate(size_t size)
		{
			if (next_byte() + size > SpaceSize)
			{
				GC<T>::Collect();
				if (next_byte() + size > SpaceSize)
					throw;									//  not enough memory
			}
			void* p = &bytes()[next_byte()];
			next_byte() += size;
			return p;
		}
		static void Deallocate(void*){}
		static void Compact()
		{
			next_byte() = 0;
			T::SPAllocator::SPIterator i;
			for (i = T::SPAllocator::iterator(); i; ++i)
			{
				T* p = i();
				void* space = Allocate(p->d_size);
				if (space < p->d_object)
					p->d_object = memcpy(space, p->d_object, p->d_size);
			}
		}
	private:
		static unsigned long& next_byte()
		{
			static unsigned long d_next_byte = 0;
			return d_next_byte;
		}
		static unsigned char* bytes()
		{
			static unsigned char d_bytes[SpaceSize];
			return d_bytes;
		}
	};
};

template <class T, template <class> class GC>
class MarkSweepObjectAllocator
{
public:
	static void* Allocate(size_t size)
	{
		return ::operator new(size);
	}
	static void Deallocate(void* object)
	{
		::operator delete(object);
	}
	static void Compact(){}
};

#endif //_POLICY_H